local L = Capitalist.Locale
local realm = GetRealmName()
local character = UnitName("player")

function Capitalist:StartPost()
	self:DebugEntry("StartPost")

	if (not self.InPostIteration) then
		ClearCursor()
		self.productsToPost = self:GetProductsInInventory()
	
		self.auctionUI.lastScanLabel:Hide()
		self.auctionUI.progressLabel:Show()
		self.auctionUI.progressLabel:SetText(L["Posting"])
		self.auctionUI.progress:Show()
		self.auctionUI.progress:SetWidth(1)
		self.auctionUI.postButton:Disable()
		self.auctionUI.scanButton:Disable()
	
		self.mode = "posting"
		self.postActive = false
		self.PostingAborted = false
		self.PostingPostActive = false
		self.PostPass = 0
		self.PostBag = 0
		self.PostSlot = 1
		self.PostPendingName = nil
		self.updateHandler:Show()
		Capitalist:OnPostIteration()
	end

	self:DebugExit("StartPost")
end

function Capitalist:GetProductsInInventory()
	self:DebugEntry("GetProductsInInventory")

	-- count the number of products in our inventory
	local numProducts = 0
	
	for bag = 0, 4 do
		for slot = 1, GetContainerNumSlots(bag) do
			local link = GetContainerItemLink(bag, slot)
			if (link) then
				local name = link:match("%[(.*)%]")
				if (self.list[name]) then
					local _, count = GetContainerItemInfo(bag, slot)
					numProducts = numProducts + count
				end
			end
		end
	end
	
	return self:DebugExit("GetProductsInInventory", numProducts)
end

function Capitalist:PostCallback(accepted)
	self:DebugEntry("PostCallback")

	self:DebugMisc(accepted)
	self:DebugMisc(self.InPostIteration)
	self:DebugMisc(self.PostingPostActive)

	local texture, count, locked, quality, readable = GetContainerItemInfo(self.PostBag, self.PostSlot)
	self:DebugMisc(texture)
	self:DebugMisc(count)
	self:DebugMisc(locked)
	self:DebugMisc(quality)
	self:DebugMisc(readable)

	local countFound = (count or 0)
	local countExpected = self.PostingPostCountAfterCreated

	-- REVIEW: When switching to multi-post, this needs to be a count==(Old-ListCount) instead of a nil check
	--if ((count == nil) or (not accepted)) then
	if ((countFound == countExpected) or (not accepted)) then
		if (self.InPostIteration) then
			self:DebugMisc("deferring")
			self:DebugMisc(accepted)
			self.PostCompleteDeferredParam = accepted
		else
			if (accepted) then
				Capitalist:PostCompleted()
			else
				Capitalist:PostAborted()
			end
			Capitalist:PostAttemptDone(accepted)
		end
	end

	self:DebugExit("PostCallback")
end

function Capitalist:PostCompleted()
	self:DebugEntry("PostCompleted")

	local productsInAH = self.db.profile.storage[realm][character].AH

	self.list[self.PostPendingName].buyoutSelf = self.list[self.PostPendingName].bestPrice

	if (not productsInAH[self.PostPendingName]) then
		productsInAH[self.PostPendingName] = 1
	else
		productsInAH[self.PostPendingName] = productsInAH[self.PostPendingName] + 1
	end

	self:DebugExit("PostCompleted")
end

function Capitalist:PostAborted()
	self:DebugEntry("PostAborted")

	-- reverse what was done in Capitalist:Post
	ClickAuctionSellItemButton()
	PickupContainerItem(self.PostBag, self.PostSlot)

	self:DebugExit("PostAborted")
end

function Capitalist:PostAttemptDone(accepted)
	self:DebugEntry("PostAttemptDone")

	self.PostPendingName = nil
	self.PostSlot = self.PostSlot + 1
	self.PostingPostCountAfterCreated = nil
	self.PostingAborted = not accepted
	self.PostingPostActive = false

	self:DebugMisc(self.productsToPost)
	self:DebugMisc(self:GetProductsInInventory())
	local alreadyPosted = self.productsToPost - self:GetProductsInInventory()
	self:DebugMisc(alreadyPosted)

	local widthNew = 100 * alreadyPosted / self.productsToPost
	self:DebugMisc(widthNew)
	self.auctionUI.progress:SetWidth(widthNew)

	Capitalist:OnPostIteration()

	self:DebugExit("PostAttemptDone")
end

function Capitalist:OnUpdatePosting()
--	self:DebugEntry("OnUpdatePosting")
	if (self.elapsed > 1) then
		self.elapsed = 0
	end
--	self:DebugExit("OnUpdatePosting")
end

function Capitalist:OnPostIteration()
	self:DebugEntry("OnPostIteration")
	self.InPostIteration = true

	self:DebugMisc("123")
	self:DebugMisc(self.PostingAborted)
	self:DebugMisc(self.PostingPostActive)
	self:DebugMisc("456")

	if ((not self.PostingAborted) and
		(not self.PostingPostActive)) then

		self:DebugMisc("P")
		self.PostingPostActive = true
		if (not self:TryPost()) then
			self:DebugMisc("p")
			self.PostingPostActive = false
		end

		if (not self.PostingPostActive) then
			self:DebugMisc(self.PostingAborted)
		end
	end

	if ((not self.PostingPostActive) or
		self.PostingAborted) then
		self:DebugMisc("Z")
		self.updateHandler:Hide()
		
		self.auctionUI.progress:Hide()
		self.auctionUI.progressLabel:Hide()
		self.auctionUI.lastScanLabel:Show()
		self.auctionUI.scanButton:Enable()
		self.auctionUI.postButton:Enable()
	end

	self.InPostIteration = false

	self:DebugMisc(self.PostCompleteDeferredParam)
	if (not self.PostCompleteDeferredParam == nil) then
		local accepted = self.PostCompleteDeferredParam
		self.PostCompleteDeferredParam = nil
		PostCallback(accepted)
	end

	self:DebugExit("OnPostIteration")
end

function Capitalist:TryPost()
	self:DebugEntry("TryPost")
	local alreadyPosted = self.productsToPost - self:GetProductsInInventory()
	self.auctionUI.progressLabel:SetText(string.format(L["Posting %d/%d"], alreadyPosted + 1, self.productsToPost))

	while (self.PostPass <= 1) do
		while (self.PostBag <= 4) do		
			self:DebugMisc(self.PostBag)
			while (self.PostSlot <= GetContainerNumSlots(self.PostBag)) do
				self:DebugMisc(self.PostSlot)
				local link = GetContainerItemLink(self.PostBag, self.PostSlot)
				if (link) then
					self:DebugMisc(link)
					local name = link:match("%[(.*)%]")
					self:DebugMisc(name)
					if (self.list[name] and
						(type(self.list[name].bestPrice) == "number")) then

						local texture, count, locked, quality, readable = GetContainerItemInfo(self.PostBag, self.PostSlot)
						self:DebugMisc(texture)
						self:DebugMisc(count)
						self:DebugMisc(locked)
						self:DebugMisc(quality)
						self:DebugMisc(readable)
						if (not locked) then
							local numRecommended = self:PostRecommended(name)

							-- For pass 0, offer to post when recommended size is > 0.
							-- For pass 1, offer to post extras (recommended size == 0).
							if ((self.PostPass == 0) ~= (numRecommended == 0)) then
								self.PostPendingName = name
								PickupContainerItem(self.PostBag, self.PostSlot)
								ClickAuctionSellItemButton()

								local numAvailable = count

								self:DebugMisc(self.list[name].bestPrice..", "..self.list[name].bestPrice..", 3, 1, "..numRecommended..", "..numAvailable)
								StartAuction(self.list[name].bestPrice, self.list[name].bestPrice, 3, 1, numRecommended, numAvailable)

								return self:DebugExit("TryPost", true)
							end
						end
					end
				end

				self.PostSlot = self.PostSlot + 1
			end

			self.PostSlot = 1
			self.PostBag = self.PostBag + 1
		end

		self.PostBag = 0
		self.PostSlot = 1
		self.PostPass = self.PostPass + 1
	end

	return self:DebugExit("TryPost", false)
end

function Capitalist:PostRecommended(productName)
	self:DebugEntry("PostRecommended")
	self:DebugMisc(productName)

	local productsInAH = self.db.profile.storage[realm][character].AH[productName] or 0
	self:DebugMisc(productsInAH)

	local skillName = self.list[productName].skillName
	self:DebugMisc(skillName)

	local profession = self:FindProfession(skillName)
	self:DebugMisc(profession)

	local product = self.db.profile.professions[profession][skillName]
	self:DebugMisc(product.count)

	local numRecommended = product.count - productsInAH
	if (numRecommended < 0) then
		numRecommended = 0
	end

	return self:DebugExit("PostRecommended", numRecommended)
end
