local L = Capitalist.Locale
local realm = GetRealmName()
local character = UnitName("player")

function Capitalist:UpdateRestockStack(profession)
	self:DebugEntry("UpdateRestockStack")

	self.inTradeskill = false

	if (not self.restock) then
		self.restock = {}
	end

	if (self.db.profile[realm] and
		self.db.profile[realm][character]) then

		if (self.db.profile[realm][character][profession]) then
			self.restock[profession] = {}

			for skillName, skillData in pairs(self.db.profile[realm][character][profession]) do
				local numCreated = skillData.numCreated or 0
				local product = self.db.profile.professions[profession][skillName]

				if (product and
					product.count and
					self:ProductIsEnabled(profession, skillName) and
					self:ProductIsVisible(profession, skillName) and
					product.count > 0) then
					local count = product.count

					local productName = skillName
					if (product.scroll) then
						productName = product.scroll
					end

					-- search AH and all inventories on this realm for product
					for player, t in pairs(self.db.profile.storage[realm]) do
						if (t.AH[productName]) then
							count = count - t.AH[productName]
						end
						-- REVIEW: is 'bank' missing here or intended/(already included)?
						if (t.inventory[productName]) then
							count = count - t.inventory[productName]
						end
					end
			
					if (count > 0) then
						self.restock[profession][skillName] = count
					end
				end
			end
	
			self.requiredComponents = {}

			for skillName, count in pairs(self.restock[profession]) do
				local product = self.db.profile.professions[profession][skillName]

				for componentName, componentCount in pairs(product.components) do
					if (self.requiredComponents[componentName] == nil) then
						self.requiredComponents[componentName] = componentCount * count
					else
						self.requiredComponents[componentName] = componentCount * count + self.requiredComponents[componentName]
					end
				end

				if (profession == L["Enchanting"]) then
					local componentName = L["Enchanting Vellum"]
					self.requiredComponents[componentName] = count;
				end
			end
		end
	end

	self:DebugExit("UpdateRestockStack")
end

function Capitalist:ShowRestockStack(profession)
	self:DebugEntry("ShowRestockStack")

	self:UpdateRestockStack(profession)

	local list = {}
	local numRestock = 0
	local components = {}
	for k, v in pairs(self.requiredComponents) do
		components[k] = v
	end
	
	for skillName, count in pairs(self.restock[profession]) do
		list[skillName] = count
		numRestock = numRestock + count
	end
	
	-- search for the required components in our inventory
	for bag = 0, 4 do
		for slot = 0, GetContainerNumSlots(bag) do
			local link = GetContainerItemLink(bag, slot)
			if (link) then
				local name = link:match("%[(.*)%]")
				if (components[name]) then
					local _, count = GetContainerItemInfo(bag, slot)
					components[name] = components[name] - count
				end
			end
		end
	end
	
	-- print out the restock stack
	if (numRestock == 0) then
		self:Print(L["Nothing to restock!"])
		return
	else
		if (numRestock == 1) then
			self:Print(L["Need to restock 1 product:"])
		else
			self:Print(string.format(L["Need to restock %d products:"], numRestock))
		end
		for skillName, count in pairs(list) do

			local skillText = skillName
			if (self.db.profile.professions and
				self.db.profile.professions[profession] and
				self.db.profile.professions[profession][skillName] and
				self.db.profile.professions[profession][skillName].link) then
				skillText = self.db.profile.professions[profession][skillName].link
			end

			ChatFrame1:AddMessage("  "..count.."x "..skillText)
		end
	end
	
	-- print out any components missing
	list = {}
	local numMissing = 0
	for componentName, componentCount in pairs(components) do
		if (componentCount > 0) then
			list[componentName] = componentCount
			numMissing = numMissing + componentCount
		end
	end
	
	if (numMissing > 0) then
		self:Print(L["Missing components:"])
		for item, count in pairs(list) do
			local itemText = item
			if (self.db.profile.components and
				self.db.profile.components[item] and
				self.db.profile.components[item].link) then
				itemText = self.db.profile.components[item].link
			end
			ChatFrame1:AddMessage("  "..count.."x "..itemText)
		end

		-- search all inventories on this realm for components
		local outputtedHeaderAltInventory
		for componentName, count in pairs(list) do
			local s
			for player, t in pairs(self.db.profile.storage[realm]) do
				if (character ~= player) then
					local c = t.inventory[componentName]
					if (c and (c > 0)) then
						if (s) then
							s = s..", x"..c.." "..player
						else
							local componentText = componentName
							if (self.db.profile.components and
								self.db.profile.components[componentName] and
								self.db.profile.components[componentName].link) then
								componentText = self.db.profile.components[componentName].link
							end
							s = "  "..componentText.."   x"..c.." "..player
						end
					end
				end
			end
			if (s and not outputtedHeaderAltInventory) then
				self:Print(L["Components in alt inventories:"])
				outputtedHeaderAltInventory = true
			end
			if (s) then
				ChatFrame1:AddMessage(s)
			end
		end

		-- search all banks on this realm for components
		local outputtedHeaderBank
		for componentName, count in pairs(list) do
			local s
			for player, t in pairs(self.db.profile.storage[realm]) do
				local c = t.bank[componentName]
				if (c and (c > 0)) then
					if (s) then
						s = s..", x"..c.." "..player
					else
						local componentText = componentName
						if (self.db.profile.components and
							self.db.profile.components[componentName] and
							self.db.profile.components[componentName].link) then
							componentText = self.db.profile.components[componentName].link
						end
						s = "  "..componentText.."   x"..c.." "..player
					end
				end
			end
			if (s and not outputtedHeaderBank) then
				self:Print(L["Components in bank:"])
				outputtedHeaderBank = true
			end
			if (s) then
				ChatFrame1:AddMessage(s)
			end
		end
	else
		self:Print(L["Everything available!"])
	end

	self:DebugExit("ShowRestockStack")
end

function Capitalist:Restock(profession)
	self:DebugEntry("Restock")

	if (self.inTradeskill) then
		return
	end

	SetTradeSkillSubClassFilter(0, 1, 1)
	SetTradeSkillInvSlotFilter(0, 1, 1)
	-- REVIEW: This had errors for non-Enchanting professions
	--ExpandTradeSkillSubClass(1)
	SetTradeSkillItemNameFilter("")
	
	for k, v in pairs(self.restock[profession]) do
		if (self:TryRestock(profession, k)) then
			return
		end
	end
	
	self:Print(L["Could not restock anything!"])

	self:DebugExit("Restock")
end

function Capitalist:TryRestock(profession, product)
	self:DebugEntry("TryRestock")
	self:DebugMisc(profession)
	self:DebugMisc(product)

	local bag
	local slot

	local isEnchanting = (profession == L["Enchanting"])

	if (isEnchanting) then
		local vellum = L["Enchanting Vellum"];

		for i = 0, 4 do
			for j = 1, GetContainerNumSlots(i) do
				local link = GetContainerItemLink(i, j)
				if (link) then
					local name = link:match("%[(.*)%]")
					if (name == vellum) then
						bag = i
						slot = j
					end
				end
			end
		end

		if (not bag) then
			return false
		end
	end

	for i = 1, GetNumTradeSkills() do
		local skill, _, numAvailable = GetTradeSkillInfo(i)
		if ((skill == product) and (numAvailable > 0)) then
			self:DebugMisc(skill)

			-- REVIEW: Opportunity for multi
			DoTradeSkill(i)
			if (isEnchanting) then
				UseContainerItem(bag, slot)
			end
			self.inTradeskill = true
			return self:DebugExit("TryRestock", true)
		end
	end

	return self:DebugExit("TryRestock", false)
end
